"""
Program that encodes and decodes string to DNA sequence
It's like base64 but for DNA and worse :D
"""

from helpers.utils import banner, convert_to_bits, convert_to_string, arguments

class DDNA:
    """
    Main class that runs the program, it's a bit messy but it works

    Works both as a standalone program and as a module
    ie: import DDNA  [in python file]
    or DDNA --help [in terminal]
    """

    def __init__(self):
        self.bases = {
            "00": "A",
            "01": "C",
            "10": "G",
            "11": "T"
        }
        self.bases_rev = {
            "A": "00",
            "C": "01",
            "G": "10",
            "T": "11"
        }

    def encode(self, string):
        """"
        Encodes String to DNA bases
        Parameters:
            String (sentences, words)

        Returns:
            DNA sequence ACTG ATAT...
        """

        bits = convert_to_bits(string)

        encoded_bits = ""
        for i in range(0, len(bits), 2):
            encoded_bits += self.bases[bits[i:i+2]]
        return encoded_bits

    def decode(self, string):
        """"
        Decodes DNA bases to String
        Parameters:
            DNA sequence ACTG ATAT...

        Returns:
            String (sentences, words)
        """
        bits = ""
        for char in string:
            bits += self.bases_rev[char]

        return convert_to_string(bits)

    def main(self):
        """
        Main function that runs the program if called as a standalone program
        """
        args = arguments()
        if args.encode:
            print(self.encode(args.encode))
        elif args.decode:
            print(self.decode(args.decode))
        else:
            banner()
            print("Usage:")
            print("\tddna -e <string>")
            print("\tddna -d <string>")
            print("\tddna --help")


if __name__ == "__main__":
    DDNA().main()



"""Utility file containg helper functions"""
import argparse


def convert_to_string(bits):
    """
    Converts bits to string
    Parametes:
        Binary bits 1's and 0's
    Returns:
        String (sentences, words)
    """
    string = ""
    for i in range(0, len(bits), 8):
        char_bin = bits[i:i+8]
        char_int = int(char_bin, 2)
        char = chr(char_int)
        string += char
    return string


def convert_to_bits(string):
    """
    Converts string to bits
    Parameters:
        String (sentences, words)

    Returns:
        Binary bits 1's and 0's
    """

    bits = ""
    for char in string:
        # convert char to int
        char_int = ord(char)
        # convert int to binary
        char_bin = bin(char_int)[2:]
        # add 0's to binary to make it 8 bits
        char_bin = char_bin.zfill(8)
        bits += char_bin
    return bits


def banner():
    """ Prints banner """

    banner_string = """
**DDNA - DNA Encoder/Decoder**

O       o O       o O       o
A O   o | A O   o | | O   o G
| G O | C 1 | O G | T | O | 1
1 o   O | T o   O A | o   O C
o       O o       O o       O

               aayushp.com.np
"""
    print(banner_string)


def arguments():
    """
    Parses arguments from terminal
    """
    parser = argparse.ArgumentParser(
        description="DDNA - DNA Encoder/Decoder")
    parser.add_argument(
        "-e", "--encode", help="Encode String to DNA", type=str)
    parser.add_argument(
        "-d", "--decode", help="Decode DNA to String", type=str)
    args = parser.parse_args()
    return args



# pylint: skip-file
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = []

setup(
    name="ddna",
    version="0.1.0",
    author="Aayush Pokharel",
    author_email="aayushpokharel36@gmail.com",
    description="Encode and decode strings to DNA",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Aayush9029/DDNA",
    py_modules=["ddna"],
    packages=find_packages(),
    install_requires=requirements,
    requires=requirements,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
